@@ -55,6 +55,10 @@ class IncompleteUpload(RuntimeError):
     pass
 
 
+class AlreadyUploaded(RuntimeError):
+    pass
+
+
 class ServiceError(RuntimeError):
     def __init__(self, message):
         super(ServiceError, self).__init__()
@@ -131,9 +135,10 @@ class _ChunksUploader:
         return True
 
     def upload_all(self):
-        for i in range(self.number_of_chunks):
-            if not self.chunks_statuses[i]["done"] and self.foreign_reference is None:
-                self.upload(i)
+        if self.foreign_reference is None:
+            for i in range(self.number_of_chunks):
+                if not self.chunks_statuses[i]["done"]:
+                    self.upload(i)
 
     def upload(self, chunk_number: int):
         self.chunks_statuses[chunk_number]["tries"] = (
@@ -226,6 +231,10 @@ class Client:
         ```origin_dir_name``` is a directory path (```dirA/dirB/dirC```). This path triggers
         the creation of the corresponding collections and sub-collections in Jama.
         """
+        upload_infos = self.upload_infos(_file_hash256(file_path))
+        if upload_infos.get("status") == "available":
+            raise AlreadyUploaded()
+
         chunked_upload = _ChunksUploader(
             file_path,
             self._endpoint + "upload/partial/",
@@ -234,9 +243,21 @@ class Client:
             file_name,
             origin_dir_name,
         )
+
+        # Resume upload across sessions
+        # Set chunk status to "done" if already available
+        # on the server
+        if upload_infos.get("status") == "incomplete":
+            for available_chunk in upload_infos.get("available_chunks", []):
+                _, chunk_number = os.path.splitext(available_chunk)[0].split("-")
+                chunk_number = int(chunk_number.lstrip("0") or "0")
+                chunked_upload.chunks_statuses[chunk_number]["done"] = True
+
         chunked_upload.upload_all()
         if not chunked_upload.is_complete:
             self.upload_status[file_path] = chunked_upload.chunks_statuses
+            # If incomplete, raise exception so the
+            # client can decide to retry or not.
             raise IncompleteUpload()
         return chunked_upload.foreign_reference
 
